
#include <X11/Xlib.h>
#include "wraster.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "bench.h"


Display *dpy;
RContext *ctx;
char *ProgName;


#if 0
void
benchmark()
{
    RImage *img;
    RColor color;
    RColor cdelta;
    double t1, t2, total, d1=0, d2=0, d3=0;
    struct timeval timev;
    int i, j;

    puts("Starting benchmark");

    gettimeofday(&timev, NULL);
    t1 = (double)timev.tv_sec + (((double)timev.tv_usec)/1000000);

    img = RCreateImage(1024, 768, True);

    gettimeofday(&timev, NULL);
    t2 = (double)timev.tv_sec + (((double)timev.tv_usec)/1000000);
    total = t2 - t1;
    printf("Image created in %f sec\n", total);

    gettimeofday(&timev, NULL);
    t1 = (double)timev.tv_sec + (((double)timev.tv_usec)/1000000);

    color.red = 0x28;
    color.green = 0x45;
    color.blue = 0x69;
    color.alpha = 0xff;
    RClearImage(img, &color);

    color.red = 0xff;
    color.green = 0x7d;
    color.blue = 0x52;
    color.alpha = 0xff;
    cdelta.red = cdelta.green = cdelta.blue = 80;
    cdelta.alpha = 0;

    gettimeofday(&timev, NULL);
    t2 = (double)timev.tv_sec + (((double)timev.tv_usec)/1000000);
    total = t2 - t1;
    printf("Image filled in %f sec\n", total);

    for(j=1; j<6; j++) {
        printf("Pass %d...\n", j);
        gettimeofday(&timev, NULL);
        t1 = (double)timev.tv_sec + (((double)timev.tv_usec)/1000000);

        color.alpha = 0xff;
        for (i=0; i< 10000; i++) {
            RDrawLine(img, 0, i%64, i%64, 63, &color);
        }

        gettimeofday(&timev, NULL);
        t2 = (double)timev.tv_sec + (((double)timev.tv_usec)/1000000);
        total = t2 - t1;
        printf("Drawing 10000 lines in %f sec\n", total);
        d1 += total;

        gettimeofday(&timev, NULL);
        t1 = (double)timev.tv_sec + (((double)timev.tv_usec)/1000000);

        color.alpha = 80;
        for (i=0; i< 10000; i++) {
            RDrawLine(img, 0, i%64, i%64, 63, &color);
        }

        gettimeofday(&timev, NULL);
        t2 = (double)timev.tv_sec + (((double)timev.tv_usec)/1000000);
        total = t2 - t1;
        printf("Drawing 10000 lines with alpha in %f sec\n", total);
        d2+=total;

        gettimeofday(&timev, NULL);
        t1 = (double)timev.tv_sec + (((double)timev.tv_usec)/1000000);

        for (i=0; i< 10000; i++) {
            ROperateLine(img, RAddOperation, 0, i%64, i%64, 63, &cdelta);
        }

        gettimeofday(&timev, NULL);
        t2 = (double)timev.tv_sec + (((double)timev.tv_usec)/1000000);
        total = t2 - t1;
        printf("Altering 10000 lines in %f sec\n", total);
        d3+=total;
    }
    printf("Average: %f, %f, %f\n", d1/5, d2/5, d3/5);

    RReleaseImage(img);
}
#else


void
benchmark()
{
    RImage *img;
    RColor color;
    RColor cdelta;
    Pixmap pix;
    int i, j;

    puts("Starting benchmark");

    puts("Creating an 1027x768 image");

    cycle_bench(1);
    img = RCreateImage(1024, 768, True);
    cycle_bench(0);

    puts("Filling image");

    cycle_bench(1);
    color.red = 0x28;
    color.green = 0x45;
    color.blue = 0x69;
    color.alpha = 0xff;
    RClearImage(img, &color);
    cycle_bench(0);

    puts("Convert image to XImage (pixmap)");
    cycle_bench(1);
    RConvertImage(ctx, img, &pix);
    cycle_bench(0);

#if 0
    color.red = 0xff;
    color.green = 0x7d;
    color.blue = 0x52;
    color.alpha = 0xff;
    cdelta.red = cdelta.green = cdelta.blue = 80;
    cdelta.alpha = 0;


    for(j=1; j<6; j++) {
        printf("Pass %d...\n", j);

        puts("Drawing 10000 lines");

        cycle_bench(1);
        color.alpha = 0xff;
        for (i=0; i< 10000; i++) {
            RDrawLine(img, 0, i%64, i%64, 63, &color);
        }
        cycle_bench(0);

        puts("Drawing 10000 lines with alpha");

        cycle_bench(1);
        color.alpha = 80;
        for (i=0; i< 10000; i++) {
            RDrawLine(img, 0, i%64, i%64, 63, &color);
        }
        cycle_bench(0);

        puts("Altering 10000 lines");

        cycle_bench(1);
        for (i=0; i< 10000; i++) {
            ROperateLine(img, RAddOperation, 0, i%64, i%64, 63, &cdelta);
        }
        cycle_bench(0);
    }
#endif

    RReleaseImage(img);
}
#endif


int main(int argc, char **argv)
{
    RContextAttributes attr;
    int visualID = -1;

    ProgName = strrchr(argv[0],'/');
    if (!ProgName)
        ProgName = argv[0];
    else
        ProgName++;

    dpy = XOpenDisplay("");
    if (!dpy) {
        puts("cant open display");
        exit(1);
    }

    attr.flags = RC_RenderMode | RC_ColorsPerChannel;

    attr.render_mode = RDitheredRendering;
    attr.colors_per_channel = 4;

    if (visualID >= 0) {
        attr.flags |= RC_VisualID;
        attr.visualid = visualID;
    }

    ctx = RCreateContext(dpy, DefaultScreen(dpy), &attr);

    if (!ctx) {
        printf("could not initialize graphics library context: %s\n",
               RMessageForError(RErrorCode));
        exit(1);
    }

    benchmark();

    return 0;
}

